@@ -12,7 +12,7 @@ pls = function(x, y, ncomp = 15, center = T, scale = F, cv = NULL,
    #   ncomp: maximum number of components to calculate
    #   center: logical, center or not x and y data
    #   scale: logical, standardize or not x data
-   #   cv: number of segments for cross-validation (1 - full CV)
+   #   cv: cross-validation settings (see \code{?crossval} for details)
    #   x.test: a matrix with predictor values for test set validation
    #   y.test: a vector with response values for test set validation
    #   method: a method to calculate PLS model
@@ -46,10 +46,15 @@ pls = function(x, y, ncomp = 15, center = T, scale = F, cv = NULL,
    # correct maximum number of components
    if (!is.null(cv))
    {
-      if (cv == 1)
+      if (!is.numeric(cv))
+         nseg = cv[[2]]
+      else
+         nseg = cv
+      
+      if (nseg == 1)
          nobj.cv = 1
       else
-         nobj.cv = ceiling(nrow(x)/cv)  
+         nobj.cv = ceiling(nrow(x)/nseg)  
    }
    else
    {   
@@ -62,8 +67,7 @@ pls = function(x, y, ncomp = 15, center = T, scale = F, cv = NULL,
    model = pls.cal(x, y, ncomp, center = center, scale = scale, method = method)
    model$alpha = alpha   
    model$calres = predict.pls(model, x, y)
-   model = selectCompNum.pls(model, ncomp)
-   
+   model = selectCompNum.pls(model, model$ncomp)
    # do cross-validation if needed
    if (!is.null(cv))
    {   
@@ -138,11 +142,10 @@ pls.cal = function(x, y, ncomp, center, scale, method = 'simpls', cv = FALSE)
    
    # do PLS
    if (method == 'simpls')
-      res = pls.simpls(x, y, ncomp)
+      res = pls.simpls(x, y, ncomp, cv = cv)
    else
       stop('Method with this name is not supported!')
 
-   
    # return a list with model parameters
    model = list(
       xloadings = res$xloadings,
@@ -156,7 +159,7 @@ pls.cal = function(x, y, ncomp, center, scale, method = 'simpls', cv = FALSE)
       xscale = attr(x, 'prep:scale'),
       ycenter = attr(y, 'prep:center'),
       yscale = attr(y, 'prep:scale'),
-      ncomp = ncomp
+      ncomp = res$ncomp
    )
    
    if (cv == FALSE)
@@ -178,6 +181,8 @@ pls.cal = function(x, y, ncomp, center, scale, method = 'simpls', cv = FALSE)
 #' a matrix with y values (responses)
 #' @param ncomp
 #' number of components to calculate
+#' @param cv
+#' logical, is model calibrated during cross-validation or not
 #' 
 #' @return 
 #' a list with computed regression coefficients, loadings and scores for x and y matrices,
@@ -187,7 +192,7 @@ pls.cal = function(x, y, ncomp, center, scale, method = 'simpls', cv = FALSE)
 #' [1]. S. de Jong. SIMPLS: An Alternative approach to partial least squares regression. 
 #' Chemometrics and Intelligent Laboratory Systems, 18, 1993 (251-263).
 #' 
-pls.simpls = function(x, y, ncomp)
+pls.simpls = function(x, y, ncomp, cv = FALSE)
 {
    x = as.matrix(x)
    y = as.matrix(y)
@@ -196,7 +201,6 @@ pls.simpls = function(x, y, ncomp)
    objnames = rownames(x);
    prednames = colnames(x);
    respnames = colnames(y);
-   compnames = paste('Comp', 1:ncomp)
    
    nobj = nrow(x)
    npred = ncol(x)
@@ -212,7 +216,7 @@ pls.simpls = function(x, y, ncomp)
    W = matrix(0, nrow = npred, ncol = ncomp)
    P = matrix(0, nrow = npred, ncol = ncomp)
    Q = matrix(0, nrow = nresp, ncol = ncomp)
-  
+   
    # loop for each components
    for (n in 1:ncomp)
    {
@@ -220,6 +224,7 @@ pls.simpls = function(x, y, ncomp)
       e = eigen(t(A) %*% A)
       q = e$vectors[1:nresp]
       
+      
       # calculate and store weights
       w = A %*% q
       c = t(w) %*% M %*% w
@@ -244,26 +249,39 @@ pls.simpls = function(x, y, ncomp)
       C = C - v %*% t(v)
       M = M - p %*% t(p)
       A = C %*% A      
+      
+      if (cv == F && e$value < 10^-12) {
+         # stop cycle is egienvalue is almost zero
+         break
+      }
    }
    
+   # truncate results if n is smaller than ncomp
+   B = B[, 1:n, , drop = F]
+   W = W[, 1:n, drop = F]
+   P = P[, 1:n, drop = F]
+   Q = Q[, 1:n, drop = F]
+   
    # calculate x and y scores
    U = y %*% Q 
    TT = x %*% (W %*% solve(t(P) %*% W))  
    
    # set names for all results
+   compnames = paste('Comp', 1:n)
    colnames(Q) = colnames(B) = colnames(P) = colnames(W) = compnames      
    colnames(TT) = colnames(U) = compnames      
    rownames(P) = rownames(B) = rownames(W) = prednames
    rownames(TT) = rownames(U) = objnames
    rownames(Q) = dimnames(B)[[3]] = respnames
-   
+
    res = list(
       coeffs = B,
       weights = W,
       xloadings = P,
       xscores = TT,
       yloadings = Q,
-      yscores = U
+      yscores = U,
+      ncomp = n
    )   
 }  
 
@@ -302,9 +320,10 @@ pls.crossval = function(model, x, y, cv, center = T, scale = F, jack.knife = T)
    
    # get matrix with indices for cv segments
    idx = crossval(nobj, cv)
-   
    seglen = ncol(idx);
-   
+   nseg = nrow(idx);
+   nrep = dim(idx)[3]
+
    yp = array(0, dim = c(nobj, ncomp, nresp))
    Q2x = matrix(0, ncol = ncomp, nrow = nobj)   
    T2x = matrix(0, ncol = ncomp, nrow = nobj)   
@@ -312,34 +331,48 @@ pls.crossval = function(model, x, y, cv, center = T, scale = F, jack.knife = T)
    T2y = matrix(0, ncol = ncomp, nrow = nobj)   
    jkcoeffs = array(0, dim = c(nvar, ncomp, ncol(y), nrow(idx)))
    
-   # loop over segments
-   for (i in 1:nrow(idx))
+   # loop over segments and repetitions
+   for (iRep in 1:nrep)
    {   
-      ind = na.exclude(idx[i,])
-      if (length(ind) > 0)
-      {   
-         xc = x[-ind, , drop = F]
-         yc = y[-ind, , drop = F]
-         xt = x[ind, , drop = F]
-         yt = y[ind, , drop = F]
+      for (iSeg in 1:nseg)
+      {         
+         ind = na.exclude(idx[iSeg, , iRep])
          
-         m = pls.cal(xc, yc, ncomp, center = center, scale = scale, cv = TRUE)               
-         res = predict.pls(m, xt, yt, cv = T)
-         
-         xdist = ldecomp.getDistances(res$xscores, m$xloadings, res$xresiduals, 
-                                      model$calres$xdecomp$tnorm)
-         ydist = ldecomp.getDistances(res$xscores, m$yloadings, res$yresiduals, 
-                                      model$calres$xdecomp$tnorm)
-         
-         yp[ind, , ] = res$yp
-         Q2x[ind, ] = xdist$Q2        
-         T2x[ind, ] = xdist$T2        
-         Q2y[ind, ] = ydist$Q2        
-         T2y[ind, ] = ydist$T2        
-         jkcoeffs[, , , i] = m$coeffs$values
+         if (length(ind) > 0)
+         {   
+            xc = x[-ind, , drop = F]
+            yc = y[-ind, , drop = F]
+            xt = x[ind, , drop = F]
+            yt = y[ind, , drop = F]
+            
+            m = pls.cal(xc, yc, ncomp, center = center, scale = scale, cv = TRUE)            
+            res = predict.pls(m, xt, yt, cv = T)
+            
+            xdist = ldecomp.getDistances(res$xscores, m$xloadings, res$xresiduals, 
+                                         model$calres$xdecomp$tnorm)
+            ydist = ldecomp.getDistances(res$xscores, m$yloadings, res$yresiduals, 
+                                         model$calres$xdecomp$tnorm)
+            
+            dim(m$coeffs$values) = c(dim(m$coeffs$values), 1)
+            
+            yp[ind, , ] = yp[ind, , , drop = F]  + res$yp
+            Q2x[ind, ]  = Q2x[ind, , drop = F] + xdist$Q2        
+            T2x[ind, ]  = T2x[ind, , drop = F] + xdist$T2        
+            Q2y[ind, ]  = ydist$Q2 + Q2y[ind, , drop = F]       
+            T2y[ind, ]  = ydist$T2 + T2y[ind, , drop = F]        
+            jkcoeffs[, , , iSeg] = jkcoeffs[, , , iSeg, drop = F] + m$coeffs$values
+         }   
       }      
    }  
    
+   # average results over repetitions
+   yp = yp / nrep
+   Q2x = Q2x / nrep
+   T2x = T2x / nrep
+   Q2y = Q2y / nrep
+   T2y = T2y / nrep
+   jkcoeffs = jkcoeffs / nrep
+   
    dimnames(yp) = list(rownames(x), colnames(model$coeffs$values), colnames(model$calres$y.ref))         
    
    res = plsres(yp, y.ref = y, ncomp.selected = model$ncomp.selected,
